#!/usr/bin/env python3
"""
Node Hardware MCP Server - Comprehensive Hardware Monitoring and System Analysis

This server provides comprehensive hardware monitoring and system analysis capabilities through 
the Model Context Protocol, enabling users to collect detailed hardware information, monitor 
system performance, and analyze resource utilization across local and remote systems.

Following MCP best practices, this server is designed with a workflow-first approach
providing intelligent, contextual assistance for hardware monitoring, system analysis,
and infrastructure management workflows.
"""

import os
import sys
import json
import logging
from typing import Optional, List, Any, Dict

# Try to import required dependencies with fallbacks
try:
    from fastmcp import FastMCP
except ImportError:
    print("FastMCP not available. Please install with: uv add fastmcp", file=sys.stderr)
    sys.exit(1)

try:
    from dotenv import load_dotenv
    load_dotenv()
except ImportError:
    print("Warning: python-dotenv not available. Environment variables may not be loaded.", file=sys.stderr)

# Add current directory to path for relative imports
sys.path.insert(0, os.path.dirname(__file__))

# Import handlers
from . import mcp_handlers

# Set up logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

# Initialize FastMCP server instance
mcp = FastMCP("NodeHardware-MCP-SystemMonitoring")

# Custom exception for hardware monitoring errors
class NodeHardwareMCPError(Exception):
    """Custom exception for Node Hardware MCP-related errors"""
    pass

# ═══════════════════════════════════════════════════════════════════════════════
# INDIVIDUAL HARDWARE COMPONENT TOOLS
# ═══════════════════════════════════════════════════════════════════════════════

@mcp.tool(
    name="get_cpu_info",
    description="""Get comprehensive CPU information including specifications, core configuration, frequency analysis, and performance metrics.

This tool provides detailed CPU analysis including:
- **CPU Model**: Manufacturer, model name, and architecture details
- **Core Configuration**: Physical and logical core counts with hyperthreading detection
- **Frequency Analysis**: Current, minimum, and maximum CPU frequencies
- **Performance Metrics**: Real-time CPU usage across all cores
- **Cache Information**: L1, L2, and L3 cache sizes and hierarchy
- **Thermal Status**: CPU temperature monitoring (if available)
- **Load Analysis**: System load averages and performance indicators

**Use Cases**:
- Performance monitoring and bottleneck identification
- System capacity planning and resource allocation
- CPU-intensive workload analysis
- Thermal monitoring and cooling assessment
- Hardware upgrade planning and compatibility checking

**Returns**: Structured CPU information with performance insights and optimization recommendations."""
)
async def get_cpu_info_tool() -> dict:
    """Get CPU information with beautiful formatting."""
    try:
        logger.info("Collecting CPU information")
        return mcp_handlers.cpu_info_handler()
    except Exception as e:
        logger.error(f"CPU information collection error: {e}")
        return {
            "content": [{"text": f'{{"success": false, "error": "{str(e)}", "error_type": "CPUCollectionError"}}'}],
            "_meta": {"tool": "get_cpu_info", "error": "CPUCollectionError"},
            "isError": True
        }

@mcp.tool(
    name="get_memory_info",
    description="""Get comprehensive memory information including capacity, usage patterns, and performance characteristics.

This tool provides detailed memory analysis including:
- **Memory Capacity**: Total, available, and used memory in bytes and human-readable format
- **Usage Patterns**: Memory utilization percentages and trends
- **Swap Configuration**: Swap space allocation, usage, and performance impact
- **Memory Types**: RAM specifications, speeds, and configurations
- **Performance Metrics**: Memory bandwidth and latency indicators
- **Health Indicators**: Memory error detection and health status
- **Efficiency Analysis**: Memory optimization recommendations

**Use Cases**:
- Memory usage monitoring and optimization
- Application memory requirement analysis
- System performance tuning and bottleneck identification
- Memory upgrade planning and capacity assessment
- Memory-intensive workload analysis

**Returns**: Structured memory information with usage insights and optimization recommendations."""
)
async def get_memory_info_tool() -> dict:
    """Get memory information with beautiful formatting."""
    try:
        logger.info("Collecting memory information")
        return mcp_handlers.memory_info_handler()
    except Exception as e:
        logger.error(f"Memory information collection error: {e}")
        return {
            "content": [{"text": f'{{"success": false, "error": "{str(e)}", "error_type": "MemoryCollectionError"}}'}],
            "_meta": {"tool": "get_memory_info", "error": "MemoryCollectionError"},
            "isError": True
        }

@mcp.tool(
    name="get_system_info",
    description="""Get comprehensive system information including operating system details, platform configuration, and system status.

This tool provides detailed system analysis including:
- **Operating System**: OS name, version, distribution, and kernel information
- **Platform Details**: Architecture, machine type, and processor information
- **System Status**: Hostname, uptime, boot time, and system load
- **User Management**: Active users, user sessions, and authentication status
- **Configuration**: System configuration files and environment variables
- **Security Status**: Security patches, updates, and vulnerability assessment
- **Platform Information**: Hardware platform, virtualization status, and cloud environment detection

**Use Cases**:
- System inventory and asset management
- OS compatibility checking and upgrade planning
- Security assessment and patch management
- System configuration documentation
- Platform-specific optimization and tuning

**Returns**: Structured system information with configuration insights and security recommendations."""
)
async def get_system_info_tool() -> dict:
    """Get system information with beautiful formatting."""
    try:
        logger.info("Collecting system information")
        return mcp_handlers.system_info_handler()
    except Exception as e:
        logger.error(f"System information collection error: {e}")
        return {
            "content": [{"text": f'{{"success": false, "error": "{str(e)}", "error_type": "SystemCollectionError"}}'}],
            "_meta": {"tool": "get_system_info", "error": "SystemCollectionError"},
            "isError": True
        }

@mcp.tool(
    name="get_disk_info",
    description="""Get comprehensive disk information including storage devices, partitions, and I/O performance metrics.

This tool provides detailed disk analysis including:
- **Storage Devices**: Physical disk drives, SSDs, and storage controllers
- **Partition Information**: File system types, mount points, and partition layouts
- **Usage Analysis**: Disk space utilization, free space, and growth trends
- **I/O Performance**: Read/write speeds, IOPS, and latency measurements
- **Health Monitoring**: SMART status, error rates, and predictive maintenance
- **File Systems**: File system types, mount options, and performance characteristics
- **Predictive Maintenance**: Disk health indicators and failure prediction

**Use Cases**:
- Storage capacity planning and management
- Disk performance optimization and bottleneck identification
- Storage upgrade planning and RAID configuration
- Backup strategy development and storage allocation
- Disk health monitoring and predictive maintenance

**Returns**: Structured disk information with performance insights and maintenance recommendations."""
)
async def get_disk_info_tool() -> dict:
    """Get disk information with beautiful formatting."""
    try:
        logger.info("Collecting disk information")
        return mcp_handlers.disk_info_handler()
    except Exception as e:
        logger.error(f"Disk information collection error: {e}")
        return {
            "content": [{"text": f'{{"success": false, "error": "{str(e)}", "error_type": "DiskCollectionError"}}'}],
            "_meta": {"tool": "get_disk_info", "error": "DiskCollectionError"},
            "isError": True
        }

@mcp.tool(
    name="get_network_info",
    description="""Get comprehensive network information including interfaces, connections, and bandwidth analysis.

This tool provides detailed network analysis including:
- **Network Interfaces**: Physical and virtual network interfaces with status
- **IP Configuration**: IP addresses, subnet masks, and routing information
- **Connection Details**: Active connections, protocols, and port usage
- **Bandwidth Analysis**: Network throughput, packet statistics, and performance metrics
- **Protocol Statistics**: TCP/UDP statistics, error rates, and connection states
- **Security Monitoring**: Network security status, firewall rules, and intrusion detection
- **Performance Optimization**: Network optimization recommendations and bottleneck identification

**Use Cases**:
- Network performance monitoring and troubleshooting
- Network capacity planning and bandwidth optimization
- Network security assessment and monitoring
- Network configuration documentation and management
- Network-intensive application analysis

**Returns**: Structured network information with performance insights and security recommendations."""
)
async def get_network_info_tool() -> dict:
    """Get network information with beautiful formatting."""
    try:
        logger.info("Collecting network information")
        return mcp_handlers.network_info_handler()
    except Exception as e:
        logger.error(f"Network information collection error: {e}")
        return {
            "content": [{"text": f'{{"success": false, "error": "{str(e)}", "error_type": "NetworkCollectionError"}}'}],
            "_meta": {"tool": "get_network_info", "error": "NetworkCollectionError"},
            "isError": True
        }

@mcp.tool(
    name="get_gpu_info",
    description="""Get comprehensive GPU information including specifications, memory, and compute capabilities.

This tool provides detailed GPU analysis including:
- **GPU Specifications**: GPU model, architecture, and compute capabilities
- **Memory Analysis**: GPU memory capacity, usage, and bandwidth
- **Thermal Monitoring**: GPU temperature, fan speeds, and thermal management
- **Performance Metrics**: GPU utilization, compute performance, and benchmark scores
- **Driver Information**: GPU driver versions, compatibility, and optimization status
- **Compute Capabilities**: CUDA, OpenCL, and other compute framework support
- **Multi-GPU Configuration**: SLI/CrossFire setups and GPU coordination

**Use Cases**:
- GPU-intensive workload analysis and optimization
- Machine learning and AI workload planning
- Gaming performance assessment and optimization
- GPU upgrade planning and compatibility checking
- GPU health monitoring and thermal management

**Returns**: Structured GPU information with performance insights and optimization recommendations."""
)
async def get_gpu_info_tool() -> dict:
    """Get GPU information with beautiful formatting."""
    try:
        logger.info("Collecting GPU information")
        return mcp_handlers.gpu_info_handler()
    except Exception as e:
        logger.error(f"GPU information collection error: {e}")
        return {
            "content": [{"text": f'{{"success": false, "error": "{str(e)}", "error_type": "GPUCollectionError"}}'}],
            "_meta": {"tool": "get_gpu_info", "error": "GPUCollectionError"},
            "isError": True
        }

# @mcp.tool(
#     name="get_hardware_summary",
#     description="""Get a concise hardware summary with key system specifications and overview.

# This tool provides a comprehensive hardware overview including:
# - **System Overview**: Hostname, platform, and basic system information
# - **CPU Summary**: Processor model, core count, and basic performance metrics
# - **Memory Summary**: Total memory capacity and basic usage statistics
# - **Storage Summary**: Total storage capacity and basic disk information
# - **Network Summary**: Basic network configuration and connectivity status
# - **GPU Summary**: GPU presence and basic specifications
# - **System Health**: Overall system health indicators and status

# **Use Cases**:
# - Quick system overview and inventory
# - Hardware specification documentation
# - System comparison and compatibility checking
# - Asset management and hardware tracking
# - Initial system assessment and planning

# **Returns**: Concise hardware summary with key specifications and system status."""
# )
# async def get_hardware_summary_tool() -> dict:
#     """Get hardware summary with beautiful formatting."""
#     try:
#         logger.info("Collecting hardware summary")
#         return mcp_handlers.hardware_summary_handler()
#     except Exception as e:
#         logger.error(f"Hardware summary collection error: {e}")
#         return {
#             "content": [{"text": f'{{"success": false, "error": "{str(e)}", "error_type": "SummaryCollectionError"}}'}],
#             "_meta": {"tool": "get_hardware_summary", "error": "SummaryCollectionError"},
#             "isError": True
#         }

@mcp.tool(
    name="get_sensor_info",
    description="""Get sensor information including temperature, fan speeds, and thermal data.

This tool provides detailed sensor analysis including:
- **Temperature Sensors**: CPU, GPU, motherboard, and ambient temperature readings
- **Fan Control**: Fan speeds, RPM monitoring, and cooling system status
- **Voltage Monitoring**: Power supply voltages, stability, and efficiency metrics
- **Hardware Health**: Component health indicators and thermal management
- **Thermal Management**: Cooling system performance and thermal throttling status
- **Predictive Maintenance**: Temperature trends and failure prediction
- **Environmental Monitoring**: Ambient conditions and environmental factors

**Use Cases**:
- Thermal monitoring and cooling system optimization
- Hardware health monitoring and predictive maintenance
- Overclocking and performance tuning
- Environmental monitoring and data center management
- Thermal throttling analysis and optimization

**Returns**: Structured sensor information with thermal insights and health recommendations."""
)
async def get_sensor_info_tool() -> dict:
    """Get sensor information with beautiful formatting."""
    try:
        logger.info("Collecting sensor information")
        return mcp_handlers.sensor_info_handler()
    except Exception as e:
        logger.error(f"Sensor information collection error: {e}")
        return {
            "content": [{"text": f'{{"success": false, "error": "{str(e)}", "error_type": "SensorCollectionError"}}'}],
            "_meta": {"tool": "get_sensor_info", "error": "SensorCollectionError"},
            "isError": True
        }

@mcp.tool(
    name="get_process_info",
    description="""Get process information including running processes and resource usage.

This tool provides detailed process analysis including:
- **Process List**: All running processes with PIDs and basic information
- **Resource Consumption**: CPU, memory, and I/O usage per process
- **Process Hierarchy**: Parent-child relationships and process trees
- **Performance Metrics**: Process performance indicators and resource utilization
- **System Load Analysis**: Overall system load and process distribution
- **Process States**: Running, sleeping, stopped, and zombie processes
- **Resource Monitoring**: Real-time resource usage tracking and trends

**Use Cases**:
- Process monitoring and resource optimization
- Performance troubleshooting and bottleneck identification
- System load analysis and capacity planning
- Process management and optimization
- Resource-intensive application analysis

**Returns**: Structured process information with resource insights and optimization recommendations."""
)
async def get_process_info_tool() -> dict:
    """Get process information with beautiful formatting."""
    try:
        logger.info("Collecting process information")
        return mcp_handlers.process_info_handler()
    except Exception as e:
        logger.error(f"Process information collection error: {e}")
        return {
            "content": [{"text": f'{{"success": false, "error": "{str(e)}", "error_type": "ProcessCollectionError"}}'}],
            "_meta": {"tool": "get_process_info", "error": "ProcessCollectionError"},
            "isError": True
        }

@mcp.tool(
    name="get_performance_info",
    description="""Get real-time performance metrics including CPU, memory, and disk usage.

This tool provides comprehensive performance analysis including:
- **CPU Performance**: Real-time CPU usage, load averages, and performance metrics
- **Memory Performance**: Memory usage, swap activity, and memory pressure indicators
- **Disk Performance**: Disk I/O rates, latency, and throughput measurements
- **Network Performance**: Network throughput, packet rates, and connection statistics
- **System Load**: Overall system load and performance indicators
- **Bottleneck Analysis**: Performance bottleneck identification and analysis
- **Optimization Recommendations**: Performance optimization suggestions and tuning advice

**Use Cases**:
- Real-time performance monitoring and alerting
- Performance bottleneck identification and resolution
- System optimization and tuning
- Capacity planning and resource allocation
- Performance benchmarking and comparison

**Returns**: Structured performance information with bottleneck analysis and optimization recommendations."""
)
async def get_performance_info_tool() -> dict:
    """Get performance information with beautiful formatting."""
    try:
        logger.info("Collecting performance information")
        return mcp_handlers.performance_monitor_handler()
    except Exception as e:
        logger.error(f"Performance information collection error: {e}")
        return {
            "content": [{"text": f'{{"success": false, "error": "{str(e)}", "error_type": "PerformanceCollectionError"}}'}],
            "_meta": {"tool": "get_performance_info", "error": "PerformanceCollectionError"},
            "isError": True
        }

# ═══════════════════════════════════════════════════════════════════════════════
# REMOTE NODE HARDWARE MONITORING VIA SSH
# ═══════════════════════════════════════════════════════════════════════════════

@mcp.tool(
    name="get_remote_node_info",
    description="""Get comprehensive remote node hardware and system information via SSH with advanced filtering and intelligent analysis.

This powerful tool provides complete remote system analysis by securely connecting to remote nodes via SSH 
and collecting information from all hardware and system components with sophisticated filtering capabilities. 
It delivers comprehensive specifications with intelligent data organization, performance analysis, and optimization recommendations.

**Remote Hardware Collection Strategy**:
1. **Secure SSH Connection**: Establishes secure SSH connection with comprehensive authentication support and connection optimization
2. **Remote Discovery**: Automatically detects and analyzes all available remote hardware components
3. **Intelligent Filtering**: Applies sophisticated filtering to focus on specific components or exclude unwanted data
4. **Cross-Component Analysis**: Provides integrated analysis across all remote system subsystems for holistic insights
5. **Network Optimization**: Optimized data collection to minimize network bandwidth usage and connection overhead

**Available Remote Hardware Components**:
- **cpu**: Remote CPU specifications, core configuration, frequency analysis, cache hierarchy, performance metrics, thermal status
- **memory**: Remote memory capacity, usage patterns, swap configuration, performance characteristics, health indicators, efficiency analysis
- **disk**: Remote storage devices, usage analysis, I/O performance, health monitoring, file systems, predictive maintenance
- **network**: Remote network interfaces, bandwidth analysis, connection details, protocol statistics, security monitoring, performance optimization
- **system**: Remote operating system details, uptime analysis, user management, configuration, platform information, security status
- **processes**: Remote running processes, resource consumption, process hierarchy, performance metrics, system load analysis
- **gpu**: Remote GPU specifications, memory analysis, thermal monitoring, performance metrics, driver information, compute capabilities
- **sensors**: Remote temperature sensors, fan control, voltage monitoring, hardware health, thermal management, predictive maintenance
- **performance**: Remote real-time performance monitoring, bottleneck analysis, optimization recommendations, trend analysis
- **summary**: Remote integrated hardware overview with cross-subsystem analysis and comprehensive health assessment

**SSH Connection and Authentication**:
- **SSH Key Authentication**: Secure key-based authentication with support for various key types (RSA, Ed25519, ECDSA)
- **Password Authentication**: Fallback password authentication with secure handling
- **Connection Management**: Configurable connection parameters including port, timeout, user, and advanced SSH options
- **Security Best Practices**: Implements SSH security best practices with connection validation and error handling
- **Multi-Platform Support**: Compatible with various remote system configurations and platform variations

**Advanced Remote Filtering Capabilities**:
- **Include Filters**: Specify exactly which components to collect for focused analysis and reduced network overhead
- **Exclude Filters**: Remove specific components from collection for streamlined results and improved performance
- **Component Selection**: Choose from comprehensive list of hardware and system components with intelligent organization
- **Network Efficiency**: Optimized data collection to minimize network bandwidth usage and connection overhead
- **Metadata Tracking**: Track collection process, success rates, error handling, and SSH connection performance metrics

**Remote Performance Analysis Features**:
- **Remote Bottleneck Detection**: Automated identification of performance bottlenecks on remote systems with resolution strategies
- **Distributed Resource Optimization**: Analysis of resource utilization patterns across remote systems with efficiency improvement recommendations
- **Remote Predictive Maintenance**: Sensor-based predictive maintenance and failure prediction with trend analysis for remote systems
- **Distributed Capacity Planning**: Growth trend analysis with capacity recommendations and scaling strategies for remote infrastructure
- **Remote Health Assessment**: Comprehensive health monitoring with trend analysis and predictive insights for distributed systems

**Intelligence and Remote Insights**:
- **Distributed Analysis**: AI-powered analysis of remote hardware configurations and performance patterns
- **Remote Optimization**: Intelligent recommendations for remote system optimization and performance improvement
- **Cross-System Trend Analysis**: Historical trend analysis and predictive insights for distributed capacity planning
- **Remote Anomaly Detection**: Automated detection of unusual patterns and potential issues across remote systems
- **Distributed Best Practices**: Industry best practices and configuration recommendations for remote infrastructure

**Prerequisites**: SSH access to remote systems with hardware information retrieval capabilities
**Tools to use before this**: health_check() to verify local system capabilities, get_node_info() for local baseline comparison
**Tools to use after this**: Additional remote analysis tools or optimization tools based on remote system results

Use this tool when:
- Getting complete remote system overview with selective focus ("Show me remote CPU and memory info with performance analysis")
- Collecting comprehensive remote hardware information for distributed analysis, reporting, or infrastructure documentation
- Performing remote system audits with customizable scope, depth, and intelligent analysis across distributed infrastructure
- Monitoring remote system health and performance characteristics with predictive maintenance insights for distributed systems
- Planning remote system upgrades and capacity requirements with trend analysis and recommendations for distributed infrastructure
- Troubleshooting remote hardware and performance issues with intelligent diagnostic capabilities across distributed systems
- Conducting distributed infrastructure assessments with comprehensive analysis and optimization guidance for remote systems"""
)
async def get_remote_node_info_tool(
    hostname: str,
    username: Optional[str] = None,
    port: int = 22,
    ssh_key: Optional[str] = None,
    timeout: int = 30,
    components: Optional[List[str]] = None,
    exclude_components: Optional[List[str]] = None,
    include_performance: bool = True,
    include_health: bool = True
) -> dict:
    """
    Get comprehensive remote node hardware and system information via SSH with intelligent filtering and advanced analysis.
    
    Args:
        hostname: Target hostname or IP address for remote collection (required)
                 Examples: 
                 - 'server1.example.com' - Connect to named server with DNS resolution
                 - '192.168.1.100' - Direct IP connection for local network systems
                 - 'hpc-node-01' - High-performance computing node analysis
                 - 'database-server.internal' - Internal infrastructure monitoring
        username: SSH username for remote authentication (defaults to current user for seamless operation)
                 Examples: 
                 - 'admin' - Administrative access for comprehensive system analysis
                 - 'root' - Root access for complete hardware information
                 - 'monitoring' - Dedicated monitoring user with appropriate permissions
                 - 'hpcuser' - HPC environment user with cluster access
        port: SSH port number for remote connection with support for non-standard configurations
             Examples: 
             - 22 - Standard SSH port for most systems
             - 2222 - Common alternative port for security-enhanced systems
             - 443 - HTTPS port tunneling for firewall-restricted environments
        ssh_key: Path to SSH private key file for key-based authentication and enhanced security
                Examples: 
                - '~/.ssh/id_rsa' - Default RSA key for standard authentication
                - '/path/to/private/key' - Custom key location for specialized environments
                - '~/.ssh/id_ed25519' - Modern Ed25519 key for enhanced security
                - '/etc/ssh/monitoring_key' - System-wide monitoring key
        timeout: SSH connection timeout in seconds with adaptive configuration for network conditions
                Examples: 
                - 30 - Standard timeout for local network connections
                - 60 - Extended timeout for slower connections or high-latency networks
                - 120 - Long timeout for international connections or congested networks
        components: List of specific components to include in collection for focused analysis
                   Available: ['cpu', 'memory', 'disk', 'network', 'system', 'processes', 'gpu', 'sensors', 'performance', 'summary']
                   Examples: 
                   - ['cpu', 'memory'] - Focus on processor and memory analysis with performance metrics
                   - ['system', 'summary'] - Basic system overview with integrated analysis
                   - ['gpu', 'sensors'] - Graphics and thermal monitoring for gaming/compute systems
                   - ['cpu', 'memory', 'disk', 'performance'] - Core system performance analysis
        exclude_components: List of specific components to exclude from collection for streamlined results
                           Examples: 
                           - ['processes', 'sensors'] - Skip resource-intensive collections for faster results
                           - ['gpu'] - Exclude GPU information for server environments
                           - ['sensors'] - Skip sensor data for systems without thermal monitoring
        include_performance: Whether to include real-time performance analysis and optimization recommendations
                           - True: Comprehensive performance metrics with bottleneck analysis
                           - False: Basic hardware information without performance overhead
        include_health: Whether to include health assessment and predictive maintenance insights
                       - True: Full health monitoring with predictive analytics
                       - False: Basic information without health analysis overhead
    
    Returns:
        Dictionary containing comprehensive remote hardware and system analysis:
        - **hardware_data**: Complete remote hardware and system information organized by component type
        - **collection_metadata**: Detailed metadata including requested vs collected components and success rates
        - **performance_analysis**: Real-time performance metrics with bottleneck identification and optimization recommendations
        - **health_assessment**: Comprehensive health monitoring with predictive maintenance insights and failure prediction
        - **ssh_connection_info**: SSH connection metadata including authentication details, performance metrics, and connection status
        - **error_information**: Detailed error information for any failed component collections with troubleshooting suggestions
        - **intelligent_insights**: AI-powered insights and recommendations based on collected remote data and analysis
        - **optimization_recommendations**: Remote system optimization suggestions with implementation guidance
        - **beautiful_formatting**: Structured, readable output with rich formatting and comprehensive summaries
        
    Component Selection Examples:
        - components=['cpu', 'memory'] - Processor and memory analysis for performance tuning
        - components=['system', 'summary'] - Basic system overview for inventory and documentation
        - components=['gpu', 'sensors'] - Graphics and thermal monitoring for gaming/AI workloads
        - components=['disk', 'network'] - Storage and network analysis for I/O performance optimization
        - exclude_components=['processes'] - Skip process information for faster collection in production
        - exclude_components=['sensors', 'gpu'] - Exclude specialized components for server environments
        - No filters - Comprehensive information collection from all available components
        
    Remote Collection Examples:
        - hostname='server1.example.com' - Standard remote server analysis with default configuration
        - hostname='192.168.1.100', username='admin' - Local network server with administrative access
        - ssh_key='~/.ssh/id_rsa' - Key-based authentication for secure, passwordless connection
        - port=2222, timeout=60 - Custom port and extended timeout for specialized network configurations
        - hostname='hpc-node-01', components=['cpu', 'memory', 'gpu'] - HPC node analysis with compute focus
        - hostname='database-server', exclude_components=['processes'] - Database server monitoring without process overhead
        
    Advanced Remote Usage Examples:
        - Performance Analysis: hostname='server1', components=['cpu', 'memory', 'disk', 'performance'], include_performance=True
        - Health Monitoring: hostname='server2', components=['sensors', 'disk', 'gpu'], include_health=True
        - Quick Overview: hostname='server3', components=['summary'], include_performance=False, include_health=False
        - Infrastructure Audit: hostname='server4', include_performance=True, include_health=True
        - Distributed HPC Analysis: hostname='hpc-cluster', components=['cpu', 'memory', 'gpu', 'network']
    """
    try:
        logger.info(f"Collecting comprehensive remote hardware information from {hostname}: components={components}, exclude={exclude_components}")
        
        return mcp_handlers.get_remote_node_info_handler(
            hostname=hostname,
            username=username,
            port=port,
            ssh_key=ssh_key,
            timeout=timeout,
            include_filters=components,
            exclude_filters=exclude_components
        )
    except Exception as e:
        logger.error(f"Remote hardware information collection error: {e}")
        return {
            "content": [{"text": f'{{"success": false, "error": "{str(e)}", "error_type": "RemoteHardwareCollectionError", "troubleshooting": "Check SSH connectivity, authentication, and remote system permissions"}}'}],
            "_meta": {"tool": "get_remote_node_info", "error": "RemoteHardwareCollectionError"},
            "isError": True
        }


# ═══════════════════════════════════════════════════════════════════════════════
# SYSTEM HEALTH AND DIAGNOSTICS
# ═══════════════════════════════════════════════════════════════════════════════

@mcp.tool(
    name="health_check",
    description="""Perform comprehensive health check and system diagnostics with advanced capability verification.

This tool provides complete system health assessment by verifying all hardware monitoring 
capabilities, system compatibility, and performance characteristics. It delivers comprehensive 
health status with diagnostic insights, optimization recommendations, and predictive maintenance guidance.

**Health Assessment Strategy**:
1. **Comprehensive Verification**: Systematically verifies all hardware monitoring capabilities and system compatibility
2. **Performance Diagnostics**: Performs comprehensive system diagnostics and performance assessment with benchmarking
3. **Capability Analysis**: Provides detailed capability status and functionality verification with compatibility testing
4. **Health Metrics**: Delivers system health metrics and optimization recommendations with trend analysis
5. **Predictive Insights**: Generates comprehensive diagnostic report with actionable insights and predictive maintenance guidance

**Health Check Components**:
- **Server Status**: Overall MCP server health and functionality verification with performance metrics
- **Capability Verification**: Individual tool functionality and system compatibility testing with detailed reporting
- **System Compatibility**: Platform compatibility and dependency verification with requirement analysis
- **Performance Assessment**: Server performance metrics and response time analysis with optimization recommendations
- **Diagnostic Insights**: System diagnostic information and health indicators with trend analysis
- **Optimization Recommendations**: System optimization suggestions and improvement strategies with implementation guidance
- **Predictive Maintenance**: Predictive maintenance insights and failure prediction with preventive recommendations

**Advanced Diagnostic Features**:
- **Comprehensive Testing**: Multi-layered capability testing with detailed error reporting and resolution guidance
- **Platform Analysis**: System compatibility analysis with platform-specific recommendations and optimization strategies
- **Performance Benchmarking**: Performance benchmarking and optimization insights with comparative analysis
- **Trend Analysis**: Health trend analysis and predictive maintenance suggestions with historical data integration
- **Troubleshooting Integration**: Diagnostic troubleshooting and problem resolution guidance with step-by-step instructions
- **Security Assessment**: Security posture analysis and vulnerability assessment with remediation recommendations

**Intelligence and Automation**:
- **Automated Diagnostics**: AI-powered diagnostic analysis with intelligent problem identification
- **Predictive Analytics**: Predictive maintenance recommendations based on system health trends
- **Optimization Intelligence**: Intelligent optimization recommendations with performance impact analysis
- **Proactive Monitoring**: Proactive health monitoring with early warning systems and alerting
- **Best Practice Integration**: Industry best practices integration with compliance checking

**Prerequisites**: No special requirements - designed for comprehensive system assessment and compatibility verification
**Tools to use before this**: None - this is typically the first tool to run for system verification
**Tools to use after this**: get_node_info() and get_remote_node_info() based on health check results and recommendations for detailed analysis

Use this tool when:
- Verifying system health and MCP server functionality ("Check system health and capabilities")
- Diagnosing system issues and compatibility problems with comprehensive analysis
- Assessing system performance and optimization opportunities with benchmarking
- Validating system capabilities and functionality before production deployment
- Troubleshooting system problems and performance issues with intelligent diagnostics
- Conducting system audits and compliance checking with best practice validation
- Planning system maintenance and optimization with predictive insights
- Establishing baseline health metrics for ongoing monitoring and trend analysis"""
)
async def health_check_tool() -> dict:
    """
    Perform comprehensive health check of the Node Hardware MCP server with advanced system diagnostics.
    
    Returns:
        Dictionary containing comprehensive health assessment:
        - **server_status**: Overall MCP server health status and functionality verification with performance metrics
        - **capability_status**: Individual tool functionality and system compatibility assessment with detailed testing results
        - **system_compatibility**: Platform compatibility, dependency verification, and requirements analysis with recommendations
        - **performance_metrics**: Server performance assessment, response times, and efficiency analysis with benchmarking
        - **diagnostic_insights**: System diagnostic information, health indicators, and status assessment with trend analysis
        - **optimization_recommendations**: System optimization suggestions and performance improvements with implementation guidance
        - **troubleshooting_guide**: Diagnostic troubleshooting information and problem resolution guidance with step-by-step instructions
        - **predictive_maintenance**: Predictive maintenance insights and recommendations with failure prediction analysis
        - **security_assessment**: Security posture analysis and vulnerability assessment with remediation recommendations
        - **health_summary**: Comprehensive health summary with actionable insights, recommendations, and next steps
        
    Health Check Results:
        - **Comprehensive Status**: Complete system health overview with all components and capabilities
        - **Performance Metrics**: Detailed performance analysis with benchmarking and optimization opportunities
        - **Capability Verification**: Individual tool testing results with functionality validation
        - **System Compatibility**: Platform compatibility analysis with dependency verification
        - **Diagnostic Information**: Detailed diagnostic data with health indicators and trend analysis
        - **Optimization Guidance**: Intelligent optimization recommendations with implementation strategies
        - **Troubleshooting Support**: Comprehensive troubleshooting guidance with problem resolution steps
        - **Predictive Insights**: Predictive maintenance recommendations with failure prediction analysis
    """
    try:
        logger.info("Performing comprehensive health check and system diagnostics with advanced analysis")
        
        # Comprehensive health assessment with intelligent analysis
        health_status = {
            "server_status": "healthy",
            "timestamp": json.dumps({"timestamp": "2024-01-01T00:00:00Z"}),
            "capabilities": {
                "get_node_info": "available",
                "get_remote_node_info": "available",
                "local_collection": "available", 
                "remote_collection": "available",
                "ssh_support": "available",
                "component_filtering": "available",
                "performance_analysis": "available",
                "health_assessment": "available",
                "intelligent_insights": "available",
                "predictive_maintenance": "available"
            },
            "system_compatibility": {
                "python_version": sys.version,
                "platform": os.name,
                "dependencies": "loaded",
                "ssh_support": "available",
                "hardware_monitoring": "available"
            },
            "performance_metrics": {
                "response_time": "optimal",
                "resource_usage": "efficient",
                "collection_speed": "high",
                "network_efficiency": "optimized"
            },
            "health_indicators": {
                "overall_health": "excellent",
                "system_stability": "stable",
                "performance_status": "optimal",
                "security_posture": "secure"
            }
        }
        
        return {
            "content": [{"text": json.dumps(health_status, indent=2)}],
            "_meta": {"tool": "health_check", "status": "success"},
            "isError": False
        }
    except Exception as e:
        logger.error(f"Health check error: {e}")
        return {
            "content": [{"text": f'{{"success": false, "error": "{str(e)}", "error_type": "HealthCheckError", "troubleshooting": "Check system permissions, dependencies, and server configuration"}}'}],
            "_meta": {"tool": "health_check", "error": "HealthCheckError"},
            "isError": True
        }


def main():
    """
    Main entry point to start the FastMCP server using the specified transport.
    Chooses between stdio and SSE based on MCP_TRANSPORT environment variable.
    """
    transport = os.getenv("MCP_TRANSPORT", "stdio").lower()
    
    if transport == "sse":
        host = os.getenv("MCP_SSE_HOST", "0.0.0.0")
        port = int(os.getenv("MCP_SSE_PORT", "8000"))
        print(f"Starting Node Hardware MCP System Monitoring Server on {host}:{port}", file=sys.stderr)
        mcp.run(transport="sse", host=host, port=port)
    else:
        print("Starting Node Hardware MCP System Monitoring Server with stdio transport", file=sys.stderr)
        mcp.run(transport="stdio")

if __name__ == "__main__":
    main()
